/*
 * Copyright (C) 2021 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "config.h"
#if ENABLE(GPU_PROCESS)
#include "GPUConnectionToWebProcess.h"

#include "ArgumentCoders.h"
#include "Decoder.h"
#include "GPUConnectionToWebProcessMessages.h"
#if ENABLE(WEBGL)
#include "GraphicsContextGLIdentifier.h"
#endif
#include "HandleMessage.h"
#include "MediaOverridesForTesting.h"
#include "RemoteAudioHardwareListenerIdentifier.h"
#if USE(AUDIO_SESSION)
#include "RemoteAudioSessionConfiguration.h"
#endif
#include "RemoteRemoteCommandListenerIdentifier.h"
#include "RemoteRenderingBackendCreationParameters.h"
#include "RenderingBackendIdentifier.h"
#if ENABLE(WEBGL)
#include "StreamConnectionBuffer.h"
#endif
#include "WebCoreArgumentCoders.h"
#if HAVE(VISIBILITY_PROPAGATION_VIEW)
#include "WebPageProxyIdentifier.h"
#endif
#if ENABLE(WEBGL)
#include <WebCore/GraphicsContextGLAttributes.h>
#endif
#include <WebCore/NowPlayingInfo.h>
#if HAVE(VISIBILITY_PROPAGATION_VIEW)
#include <WebCore/PageIdentifier.h>
#endif
#include <wtf/Vector.h>
#include <wtf/text/WTFString.h>

namespace Messages {

namespace GPUConnectionToWebProcess {

#if USE(AUDIO_SESSION)

void EnsureAudioSession::send(UniqueRef<IPC::Encoder>&& encoder, IPC::Connection& connection, const WebKit::RemoteAudioSessionConfiguration& configuration)
{
    encoder.get() << configuration;
    connection.sendSyncReply(WTFMove(encoder));
}

#endif

} // namespace GPUConnectionToWebProcess

} // namespace Messages

namespace WebKit {

void GPUConnectionToWebProcess::didReceiveMessage(IPC::Connection& connection, IPC::Decoder& decoder)
{
    auto protectedThis = makeRef(*this);
    if (decoder.messageName() == Messages::GPUConnectionToWebProcess::CreateRenderingBackend::name())
        return IPC::handleMessage<Messages::GPUConnectionToWebProcess::CreateRenderingBackend>(decoder, this, &GPUConnectionToWebProcess::createRenderingBackend);
    if (decoder.messageName() == Messages::GPUConnectionToWebProcess::ReleaseRenderingBackend::name())
        return IPC::handleMessage<Messages::GPUConnectionToWebProcess::ReleaseRenderingBackend>(decoder, this, &GPUConnectionToWebProcess::releaseRenderingBackend);
#if ENABLE(WEBGL)
    if (decoder.messageName() == Messages::GPUConnectionToWebProcess::CreateGraphicsContextGL::name())
        return IPC::handleMessage<Messages::GPUConnectionToWebProcess::CreateGraphicsContextGL>(decoder, this, &GPUConnectionToWebProcess::createGraphicsContextGL);
#endif
#if ENABLE(WEBGL)
    if (decoder.messageName() == Messages::GPUConnectionToWebProcess::ReleaseGraphicsContextGL::name())
        return IPC::handleMessage<Messages::GPUConnectionToWebProcess::ReleaseGraphicsContextGL>(decoder, this, &GPUConnectionToWebProcess::releaseGraphicsContextGL);
#endif
    if (decoder.messageName() == Messages::GPUConnectionToWebProcess::ClearNowPlayingInfo::name())
        return IPC::handleMessage<Messages::GPUConnectionToWebProcess::ClearNowPlayingInfo>(decoder, this, &GPUConnectionToWebProcess::clearNowPlayingInfo);
    if (decoder.messageName() == Messages::GPUConnectionToWebProcess::SetNowPlayingInfo::name())
        return IPC::handleMessage<Messages::GPUConnectionToWebProcess::SetNowPlayingInfo>(decoder, this, &GPUConnectionToWebProcess::setNowPlayingInfo);
#if PLATFORM(IOS_FAMILY)
    if (decoder.messageName() == Messages::GPUConnectionToWebProcess::EnsureMediaSessionHelper::name())
        return IPC::handleMessage<Messages::GPUConnectionToWebProcess::EnsureMediaSessionHelper>(decoder, this, &GPUConnectionToWebProcess::ensureMediaSessionHelper);
#endif
#if ENABLE(VP9)
    if (decoder.messageName() == Messages::GPUConnectionToWebProcess::EnableVP9Decoders::name())
        return IPC::handleMessage<Messages::GPUConnectionToWebProcess::EnableVP9Decoders>(decoder, this, &GPUConnectionToWebProcess::enableVP9Decoders);
#endif
#if HAVE(VISIBILITY_PROPAGATION_VIEW)
    if (decoder.messageName() == Messages::GPUConnectionToWebProcess::CreateVisibilityPropagationContextForPage::name())
        return IPC::handleMessage<Messages::GPUConnectionToWebProcess::CreateVisibilityPropagationContextForPage>(decoder, this, &GPUConnectionToWebProcess::createVisibilityPropagationContextForPage);
#endif
#if HAVE(VISIBILITY_PROPAGATION_VIEW)
    if (decoder.messageName() == Messages::GPUConnectionToWebProcess::DestroyVisibilityPropagationContextForPage::name())
        return IPC::handleMessage<Messages::GPUConnectionToWebProcess::DestroyVisibilityPropagationContextForPage>(decoder, this, &GPUConnectionToWebProcess::destroyVisibilityPropagationContextForPage);
#endif
    if (decoder.messageName() == Messages::GPUConnectionToWebProcess::SetMediaOverridesForTesting::name())
        return IPC::handleMessage<Messages::GPUConnectionToWebProcess::SetMediaOverridesForTesting>(decoder, this, &GPUConnectionToWebProcess::setMediaOverridesForTesting);
    if (decoder.messageName() == Messages::GPUConnectionToWebProcess::CreateAudioHardwareListener::name())
        return IPC::handleMessage<Messages::GPUConnectionToWebProcess::CreateAudioHardwareListener>(decoder, this, &GPUConnectionToWebProcess::createAudioHardwareListener);
    if (decoder.messageName() == Messages::GPUConnectionToWebProcess::ReleaseAudioHardwareListener::name())
        return IPC::handleMessage<Messages::GPUConnectionToWebProcess::ReleaseAudioHardwareListener>(decoder, this, &GPUConnectionToWebProcess::releaseAudioHardwareListener);
    if (decoder.messageName() == Messages::GPUConnectionToWebProcess::CreateRemoteCommandListener::name())
        return IPC::handleMessage<Messages::GPUConnectionToWebProcess::CreateRemoteCommandListener>(decoder, this, &GPUConnectionToWebProcess::createRemoteCommandListener);
    if (decoder.messageName() == Messages::GPUConnectionToWebProcess::ReleaseRemoteCommandListener::name())
        return IPC::handleMessage<Messages::GPUConnectionToWebProcess::ReleaseRemoteCommandListener>(decoder, this, &GPUConnectionToWebProcess::releaseRemoteCommandListener);
    if (decoder.messageName() == Messages::GPUConnectionToWebProcess::SetUserPreferredLanguages::name())
        return IPC::handleMessage<Messages::GPUConnectionToWebProcess::SetUserPreferredLanguages>(decoder, this, &GPUConnectionToWebProcess::setUserPreferredLanguages);
    if (decoder.messageName() == Messages::GPUConnectionToWebProcess::ConfigureLoggingChannel::name())
        return IPC::handleMessage<Messages::GPUConnectionToWebProcess::ConfigureLoggingChannel>(decoder, this, &GPUConnectionToWebProcess::configureLoggingChannel);
    if (dispatchMessage(connection, decoder))
        return;
    UNUSED_PARAM(connection);
    UNUSED_PARAM(decoder);
#if ENABLE(IPC_TESTING_API)
    if (connection.ignoreInvalidMessageForTesting())
        return;
#endif // ENABLE(IPC_TESTING_API)
    ASSERT_NOT_REACHED_WITH_MESSAGE("Unhandled message %s to %" PRIu64, description(decoder.messageName()), decoder.destinationID());
}

bool GPUConnectionToWebProcess::didReceiveSyncMessage(IPC::Connection& connection, IPC::Decoder& decoder, UniqueRef<IPC::Encoder>& replyEncoder)
{
    auto protectedThis = makeRef(*this);
#if USE(AUDIO_SESSION)
    if (decoder.messageName() == Messages::GPUConnectionToWebProcess::EnsureAudioSession::name())
        return IPC::handleMessageSynchronous<Messages::GPUConnectionToWebProcess::EnsureAudioSession>(connection, decoder, replyEncoder, this, &GPUConnectionToWebProcess::ensureAudioSession);
#endif
    if (dispatchSyncMessage(connection, decoder, replyEncoder))
        return true;
    UNUSED_PARAM(connection);
    UNUSED_PARAM(decoder);
    UNUSED_PARAM(replyEncoder);
#if ENABLE(IPC_TESTING_API)
    if (connection.ignoreInvalidMessageForTesting())
        return false;
#endif // ENABLE(IPC_TESTING_API)
    ASSERT_NOT_REACHED_WITH_MESSAGE("Unhandled synchronous message %s to %" PRIu64, description(decoder.messageName()), decoder.destinationID());
    return false;
}

} // namespace WebKit

#endif // ENABLE(GPU_PROCESS)
