/*
 * Copyright (C) 2021 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "config.h"
#if ENABLE(GPU_PROCESS)
#include "GPUProcessConnection.h"

#if ENABLE(ROUTING_ARBITRATION)
#include "AudioSessionRoutingArbitratorProxy.h"
#endif
#include "Decoder.h"
#include "GPUProcessConnectionMessages.h"
#include "HandleMessage.h"
#include "WebCoreArgumentCoders.h"
#if ENABLE(ROUTING_ARBITRATION)
#include <WebCore/AudioSession.h>
#endif
#include <WebCore/PlatformMediaSession.h>

namespace Messages {

namespace GPUProcessConnection {

#if ENABLE(ROUTING_ARBITRATION)

void BeginRoutingArbitrationWithCategory::callReply(IPC::Decoder& decoder, CompletionHandler<void(WebKit::AudioSessionRoutingArbitratorProxy::RoutingArbitrationError&&, WebKit::AudioSessionRoutingArbitratorProxy::DefaultRouteChanged&&)>&& completionHandler)
{
    std::optional<WebKit::AudioSessionRoutingArbitratorProxy::RoutingArbitrationError> error;
    decoder >> error;
    if (!error) {
        ASSERT_NOT_REACHED();
        cancelReply(WTFMove(completionHandler));
        return;
    }
    std::optional<WebKit::AudioSessionRoutingArbitratorProxy::DefaultRouteChanged> defaultRouteChanged;
    decoder >> defaultRouteChanged;
    if (!defaultRouteChanged) {
        ASSERT_NOT_REACHED();
        cancelReply(WTFMove(completionHandler));
        return;
    }
    completionHandler(WTFMove(*error), WTFMove(*defaultRouteChanged));
}

void BeginRoutingArbitrationWithCategory::cancelReply(CompletionHandler<void(WebKit::AudioSessionRoutingArbitratorProxy::RoutingArbitrationError&&, WebKit::AudioSessionRoutingArbitratorProxy::DefaultRouteChanged&&)>&& completionHandler)
{
    completionHandler(IPC::AsyncReplyError<WebKit::AudioSessionRoutingArbitratorProxy::RoutingArbitrationError>::create(), IPC::AsyncReplyError<WebKit::AudioSessionRoutingArbitratorProxy::DefaultRouteChanged>::create());
}

void BeginRoutingArbitrationWithCategory::send(UniqueRef<IPC::Encoder>&& encoder, IPC::Connection& connection, WebKit::AudioSessionRoutingArbitratorProxy::RoutingArbitrationError error, WebKit::AudioSessionRoutingArbitratorProxy::DefaultRouteChanged defaultRouteChanged)
{
    encoder.get() << error;
    encoder.get() << defaultRouteChanged;
    connection.sendSyncReply(WTFMove(encoder));
}

#endif

} // namespace GPUProcessConnection

} // namespace Messages

namespace WebKit {

void GPUProcessConnection::didReceiveMessage(IPC::Connection& connection, IPC::Decoder& decoder)
{
    auto protectedThis = makeRef(*this);
    if (decoder.messageName() == Messages::GPUProcessConnection::DidReceiveRemoteCommand::name())
        return IPC::handleMessage<Messages::GPUProcessConnection::DidReceiveRemoteCommand>(decoder, this, &GPUProcessConnection::didReceiveRemoteCommand);
#if ENABLE(ROUTING_ARBITRATION)
    if (decoder.messageName() == Messages::GPUProcessConnection::BeginRoutingArbitrationWithCategory::name())
        return IPC::handleMessageAsync<Messages::GPUProcessConnection::BeginRoutingArbitrationWithCategory>(connection, decoder, this, &GPUProcessConnection::beginRoutingArbitrationWithCategory);
#endif
#if ENABLE(ROUTING_ARBITRATION)
    if (decoder.messageName() == Messages::GPUProcessConnection::EndRoutingArbitration::name())
        return IPC::handleMessage<Messages::GPUProcessConnection::EndRoutingArbitration>(decoder, this, &GPUProcessConnection::endRoutingArbitration);
#endif
    if (dispatchMessage(connection, decoder))
        return;
    UNUSED_PARAM(connection);
    UNUSED_PARAM(decoder);
#if ENABLE(IPC_TESTING_API)
    if (connection.ignoreInvalidMessageForTesting())
        return;
#endif // ENABLE(IPC_TESTING_API)
    ASSERT_NOT_REACHED_WITH_MESSAGE("Unhandled message %s to %" PRIu64, description(decoder.messageName()), decoder.destinationID());
}

bool GPUProcessConnection::didReceiveSyncMessage(IPC::Connection& connection, IPC::Decoder& decoder, UniqueRef<IPC::Encoder>& replyEncoder)
{
    auto protectedThis = makeRef(*this);
    if (dispatchSyncMessage(connection, decoder, replyEncoder))
        return true;
    UNUSED_PARAM(connection);
    UNUSED_PARAM(decoder);
    UNUSED_PARAM(replyEncoder);
#if ENABLE(IPC_TESTING_API)
    if (connection.ignoreInvalidMessageForTesting())
        return false;
#endif // ENABLE(IPC_TESTING_API)
    ASSERT_NOT_REACHED_WITH_MESSAGE("Unhandled synchronous message %s to %" PRIu64, description(decoder.messageName()), decoder.destinationID());
    return false;
}

} // namespace WebKit

#endif // ENABLE(GPU_PROCESS)
