/*
 * Copyright (C) 2021 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "config.h"
#if ENABLE(GPU_PROCESS)
#include "GPUProcess.h"

#include "ArgumentCoders.h"
#include "Attachment.h"
#include "Decoder.h"
#include "GPUProcessConnectionParameters.h"
#include "GPUProcessCreationParameters.h"
#include "GPUProcessMessages.h"
#include "GPUProcessSessionParameters.h"
#include "HandleMessage.h"
#if ENABLE(MEDIA_STREAM)
#include "SandboxExtension.h"
#endif
#include "WebCoreArgumentCoders.h"
#if ENABLE(MEDIA_STREAM)
#include <WebCore/MockMediaDevice.h>
#endif
#include <WebCore/ProcessIdentifier.h>
#if ENABLE(MEDIA_STREAM)
#include <WebCore/SecurityOriginData.h>
#endif
#include <optional>
#include <pal/SessionID.h>
#if ENABLE(MEDIA_STREAM)
#include <wtf/Vector.h>
#endif
#if ENABLE(MEDIA_STREAM)
#include <wtf/text/WTFString.h>
#endif

namespace Messages {

namespace GPUProcess {

void CreateGPUConnectionToWebProcess::callReply(IPC::Decoder& decoder, CompletionHandler<void(std::optional<IPC::Attachment>&&)>&& completionHandler)
{
    std::optional<std::optional<IPC::Attachment>> connectionIdentifier;
    decoder >> connectionIdentifier;
    if (!connectionIdentifier) {
        ASSERT_NOT_REACHED();
        cancelReply(WTFMove(completionHandler));
        return;
    }
    completionHandler(WTFMove(*connectionIdentifier));
}

void CreateGPUConnectionToWebProcess::cancelReply(CompletionHandler<void(std::optional<IPC::Attachment>&&)>&& completionHandler)
{
    completionHandler(IPC::AsyncReplyError<std::optional<IPC::Attachment>>::create());
}

void CreateGPUConnectionToWebProcess::send(UniqueRef<IPC::Encoder>&& encoder, IPC::Connection& connection, const std::optional<IPC::Attachment>& connectionIdentifier)
{
    encoder.get() << connectionIdentifier;
    connection.sendSyncReply(WTFMove(encoder));
}

void PrepareToSuspend::callReply(IPC::Decoder& decoder, CompletionHandler<void()>&& completionHandler)
{
    completionHandler();
}

void PrepareToSuspend::cancelReply(CompletionHandler<void()>&& completionHandler)
{
    completionHandler();
}

void PrepareToSuspend::send(UniqueRef<IPC::Encoder>&& encoder, IPC::Connection& connection)
{
    connection.sendSyncReply(WTFMove(encoder));
}

#if ENABLE(MEDIA_STREAM)

void UpdateCaptureAccess::callReply(IPC::Decoder& decoder, CompletionHandler<void()>&& completionHandler)
{
    completionHandler();
}

void UpdateCaptureAccess::cancelReply(CompletionHandler<void()>&& completionHandler)
{
    completionHandler();
}

void UpdateCaptureAccess::send(UniqueRef<IPC::Encoder>&& encoder, IPC::Connection& connection)
{
    connection.sendSyncReply(WTFMove(encoder));
}

#endif

} // namespace GPUProcess

} // namespace Messages

namespace WebKit {

void GPUProcess::didReceiveGPUProcessMessage(IPC::Connection& connection, IPC::Decoder& decoder)
{
    auto protectedThis = makeRef(*this);
    if (decoder.messageName() == Messages::GPUProcess::InitializeGPUProcess::name())
        return IPC::handleMessage<Messages::GPUProcess::InitializeGPUProcess>(decoder, this, &GPUProcess::initializeGPUProcess);
    if (decoder.messageName() == Messages::GPUProcess::CreateGPUConnectionToWebProcess::name())
        return IPC::handleMessageAsync<Messages::GPUProcess::CreateGPUConnectionToWebProcess>(connection, decoder, this, &GPUProcess::createGPUConnectionToWebProcess);
    if (decoder.messageName() == Messages::GPUProcess::PrepareToSuspend::name())
        return IPC::handleMessageAsync<Messages::GPUProcess::PrepareToSuspend>(connection, decoder, this, &GPUProcess::prepareToSuspend);
    if (decoder.messageName() == Messages::GPUProcess::ProcessDidResume::name())
        return IPC::handleMessage<Messages::GPUProcess::ProcessDidResume>(decoder, this, &GPUProcess::processDidResume);
    if (decoder.messageName() == Messages::GPUProcess::AddSession::name())
        return IPC::handleMessage<Messages::GPUProcess::AddSession>(decoder, this, &GPUProcess::addSession);
    if (decoder.messageName() == Messages::GPUProcess::RemoveSession::name())
        return IPC::handleMessage<Messages::GPUProcess::RemoveSession>(decoder, this, &GPUProcess::removeSession);
#if ENABLE(MEDIA_STREAM)
    if (decoder.messageName() == Messages::GPUProcess::SetMockCaptureDevicesEnabled::name())
        return IPC::handleMessage<Messages::GPUProcess::SetMockCaptureDevicesEnabled>(decoder, this, &GPUProcess::setMockCaptureDevicesEnabled);
#endif
#if ENABLE(MEDIA_STREAM)
    if (decoder.messageName() == Messages::GPUProcess::SetOrientationForMediaCapture::name())
        return IPC::handleMessage<Messages::GPUProcess::SetOrientationForMediaCapture>(decoder, this, &GPUProcess::setOrientationForMediaCapture);
#endif
#if ENABLE(MEDIA_STREAM)
    if (decoder.messageName() == Messages::GPUProcess::UpdateSandboxAccess::name())
        return IPC::handleMessage<Messages::GPUProcess::UpdateSandboxAccess>(decoder, this, &GPUProcess::updateSandboxAccess);
#endif
#if ENABLE(MEDIA_STREAM)
    if (decoder.messageName() == Messages::GPUProcess::UpdateCaptureAccess::name())
        return IPC::handleMessageAsync<Messages::GPUProcess::UpdateCaptureAccess>(connection, decoder, this, &GPUProcess::updateCaptureAccess);
#endif
#if ENABLE(MEDIA_STREAM)
    if (decoder.messageName() == Messages::GPUProcess::UpdateCaptureOrigin::name())
        return IPC::handleMessage<Messages::GPUProcess::UpdateCaptureOrigin>(decoder, this, &GPUProcess::updateCaptureOrigin);
#endif
#if ENABLE(MEDIA_STREAM)
    if (decoder.messageName() == Messages::GPUProcess::AddMockMediaDevice::name())
        return IPC::handleMessage<Messages::GPUProcess::AddMockMediaDevice>(decoder, this, &GPUProcess::addMockMediaDevice);
#endif
#if ENABLE(MEDIA_STREAM)
    if (decoder.messageName() == Messages::GPUProcess::ClearMockMediaDevices::name())
        return IPC::handleMessage<Messages::GPUProcess::ClearMockMediaDevices>(decoder, this, &GPUProcess::clearMockMediaDevices);
#endif
#if ENABLE(MEDIA_STREAM)
    if (decoder.messageName() == Messages::GPUProcess::RemoveMockMediaDevice::name())
        return IPC::handleMessage<Messages::GPUProcess::RemoveMockMediaDevice>(decoder, this, &GPUProcess::removeMockMediaDevice);
#endif
#if ENABLE(MEDIA_STREAM)
    if (decoder.messageName() == Messages::GPUProcess::ResetMockMediaDevices::name())
        return IPC::handleMessage<Messages::GPUProcess::ResetMockMediaDevices>(decoder, this, &GPUProcess::resetMockMediaDevices);
#endif
#if PLATFORM(MAC)
    if (decoder.messageName() == Messages::GPUProcess::DisplayConfigurationChanged::name())
        return IPC::handleMessage<Messages::GPUProcess::DisplayConfigurationChanged>(decoder, this, &GPUProcess::displayConfigurationChanged);
#endif
#if ENABLE(MEDIA_SOURCE)
    if (decoder.messageName() == Messages::GPUProcess::SetWebMParserEnabled::name())
        return IPC::handleMessage<Messages::GPUProcess::SetWebMParserEnabled>(decoder, this, &GPUProcess::setWebMParserEnabled);
#endif
#if ENABLE(WEBM_FORMAT_READER)
    if (decoder.messageName() == Messages::GPUProcess::SetWebMFormatReaderEnabled::name())
        return IPC::handleMessage<Messages::GPUProcess::SetWebMFormatReaderEnabled>(decoder, this, &GPUProcess::setWebMFormatReaderEnabled);
#endif
#if ENABLE(OPUS)
    if (decoder.messageName() == Messages::GPUProcess::SetOpusDecoderEnabled::name())
        return IPC::handleMessage<Messages::GPUProcess::SetOpusDecoderEnabled>(decoder, this, &GPUProcess::setOpusDecoderEnabled);
#endif
#if ENABLE(VORBIS)
    if (decoder.messageName() == Messages::GPUProcess::SetVorbisDecoderEnabled::name())
        return IPC::handleMessage<Messages::GPUProcess::SetVorbisDecoderEnabled>(decoder, this, &GPUProcess::setVorbisDecoderEnabled);
#endif
    UNUSED_PARAM(connection);
    UNUSED_PARAM(decoder);
#if ENABLE(IPC_TESTING_API)
    if (connection.ignoreInvalidMessageForTesting())
        return;
#endif // ENABLE(IPC_TESTING_API)
    ASSERT_NOT_REACHED_WITH_MESSAGE("Unhandled message %s to %" PRIu64, description(decoder.messageName()), decoder.destinationID());
}

} // namespace WebKit

#endif // ENABLE(GPU_PROCESS)
