/*
 * Copyright (C) 2021 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "config.h"
#if ENABLE(GPU_PROCESS)
#include "MediaPlayerPrivateRemote.h"

#include "ArgumentCoders.h"
#include "DataReference.h"
#include "Decoder.h"
#include "HandleMessage.h"
#include "MediaPlayerPrivateRemoteMessages.h"
#include "RemoteMediaPlayerState.h"
#include "RemoteMediaResourceIdentifier.h"
#include "TextTrackPrivateRemoteConfiguration.h"
#include "TrackPrivateRemoteConfiguration.h"
#include "TrackPrivateRemoteIdentifier.h"
#include "WebCoreArgumentCoders.h"
#if PLATFORM(IOS_FAMILY)
#include <WebCore/Cookie.h>
#endif
#include <WebCore/FloatSize.h>
#include <WebCore/ISOVTTCue.h>
#include <WebCore/InbandGenericCue.h>
#include <WebCore/PlatformMediaResourceLoader.h>
#include <WebCore/ResourceError.h>
#include <WebCore/ResourceRequest.h>
#if ENABLE(DATACUE_VALUE)
#include <WebCore/SerializedPlatformDataCueValue.h>
#endif
#include <wtf/Expected.h>
#include <wtf/MediaTime.h>
#include <wtf/MonotonicTime.h>
#include <wtf/Seconds.h>
#include <wtf/URLHash.h>
#if PLATFORM(IOS_FAMILY)
#include <wtf/Vector.h>
#endif
#include <wtf/text/WTFString.h>

namespace Messages {

namespace MediaPlayerPrivateRemote {

void RequestResource::callReply(IPC::Decoder& decoder, CompletionHandler<void()>&& completionHandler)
{
    completionHandler();
}

void RequestResource::cancelReply(CompletionHandler<void()>&& completionHandler)
{
    completionHandler();
}

void RequestResource::send(UniqueRef<IPC::Encoder>&& encoder, IPC::Connection& connection)
{
    connection.sendSyncReply(WTFMove(encoder));
}

void SendH2Ping::callReply(IPC::Decoder& decoder, CompletionHandler<void(Expected<Seconds, WebCore::ResourceError>&&)>&& completionHandler)
{
    std::optional<Expected<Seconds, WebCore::ResourceError>> result;
    decoder >> result;
    if (!result) {
        ASSERT_NOT_REACHED();
        cancelReply(WTFMove(completionHandler));
        return;
    }
    completionHandler(WTFMove(*result));
}

void SendH2Ping::cancelReply(CompletionHandler<void(Expected<Seconds, WebCore::ResourceError>&&)>&& completionHandler)
{
    completionHandler(IPC::AsyncReplyError<Expected<Seconds, WebCore::ResourceError>>::create());
}

void SendH2Ping::send(UniqueRef<IPC::Encoder>&& encoder, IPC::Connection& connection, const Expected<Seconds, WebCore::ResourceError>& result)
{
    encoder.get() << result;
    connection.sendSyncReply(WTFMove(encoder));
}

#if PLATFORM(IOS_FAMILY)

void GetRawCookies::callReply(IPC::Decoder& decoder, CompletionHandler<void(Vector<WebCore::Cookie>&&)>&& completionHandler)
{
    std::optional<Vector<WebCore::Cookie>> result;
    decoder >> result;
    if (!result) {
        ASSERT_NOT_REACHED();
        cancelReply(WTFMove(completionHandler));
        return;
    }
    completionHandler(WTFMove(*result));
}

void GetRawCookies::cancelReply(CompletionHandler<void(Vector<WebCore::Cookie>&&)>&& completionHandler)
{
    completionHandler(IPC::AsyncReplyError<Vector<WebCore::Cookie>>::create());
}

void GetRawCookies::send(UniqueRef<IPC::Encoder>&& encoder, IPC::Connection& connection, const Vector<WebCore::Cookie>& result)
{
    encoder.get() << result;
    connection.sendSyncReply(WTFMove(encoder));
}

#endif

} // namespace MediaPlayerPrivateRemote

} // namespace Messages

namespace WebKit {

void MediaPlayerPrivateRemote::didReceiveMessage(IPC::Connection& connection, IPC::Decoder& decoder)
{
    if (decoder.messageName() == Messages::MediaPlayerPrivateRemote::NetworkStateChanged::name())
        return IPC::handleMessage<Messages::MediaPlayerPrivateRemote::NetworkStateChanged>(decoder, this, &MediaPlayerPrivateRemote::networkStateChanged);
    if (decoder.messageName() == Messages::MediaPlayerPrivateRemote::ReadyStateChanged::name())
        return IPC::handleMessage<Messages::MediaPlayerPrivateRemote::ReadyStateChanged>(decoder, this, &MediaPlayerPrivateRemote::readyStateChanged);
    if (decoder.messageName() == Messages::MediaPlayerPrivateRemote::FirstVideoFrameAvailable::name())
        return IPC::handleMessage<Messages::MediaPlayerPrivateRemote::FirstVideoFrameAvailable>(decoder, this, &MediaPlayerPrivateRemote::firstVideoFrameAvailable);
    if (decoder.messageName() == Messages::MediaPlayerPrivateRemote::VolumeChanged::name())
        return IPC::handleMessage<Messages::MediaPlayerPrivateRemote::VolumeChanged>(decoder, this, &MediaPlayerPrivateRemote::volumeChanged);
    if (decoder.messageName() == Messages::MediaPlayerPrivateRemote::MuteChanged::name())
        return IPC::handleMessage<Messages::MediaPlayerPrivateRemote::MuteChanged>(decoder, this, &MediaPlayerPrivateRemote::muteChanged);
    if (decoder.messageName() == Messages::MediaPlayerPrivateRemote::TimeChanged::name())
        return IPC::handleMessage<Messages::MediaPlayerPrivateRemote::TimeChanged>(decoder, this, &MediaPlayerPrivateRemote::timeChanged);
    if (decoder.messageName() == Messages::MediaPlayerPrivateRemote::DurationChanged::name())
        return IPC::handleMessage<Messages::MediaPlayerPrivateRemote::DurationChanged>(decoder, this, &MediaPlayerPrivateRemote::durationChanged);
    if (decoder.messageName() == Messages::MediaPlayerPrivateRemote::RateChanged::name())
        return IPC::handleMessage<Messages::MediaPlayerPrivateRemote::RateChanged>(decoder, this, &MediaPlayerPrivateRemote::rateChanged);
    if (decoder.messageName() == Messages::MediaPlayerPrivateRemote::PlaybackStateChanged::name())
        return IPC::handleMessage<Messages::MediaPlayerPrivateRemote::PlaybackStateChanged>(decoder, this, &MediaPlayerPrivateRemote::playbackStateChanged);
    if (decoder.messageName() == Messages::MediaPlayerPrivateRemote::EngineFailedToLoad::name())
        return IPC::handleMessage<Messages::MediaPlayerPrivateRemote::EngineFailedToLoad>(decoder, this, &MediaPlayerPrivateRemote::engineFailedToLoad);
    if (decoder.messageName() == Messages::MediaPlayerPrivateRemote::UpdateCachedState::name())
        return IPC::handleMessage<Messages::MediaPlayerPrivateRemote::UpdateCachedState>(decoder, this, &MediaPlayerPrivateRemote::updateCachedState);
    if (decoder.messageName() == Messages::MediaPlayerPrivateRemote::CharacteristicChanged::name())
        return IPC::handleMessage<Messages::MediaPlayerPrivateRemote::CharacteristicChanged>(decoder, this, &MediaPlayerPrivateRemote::characteristicChanged);
    if (decoder.messageName() == Messages::MediaPlayerPrivateRemote::SizeChanged::name())
        return IPC::handleMessage<Messages::MediaPlayerPrivateRemote::SizeChanged>(decoder, this, &MediaPlayerPrivateRemote::sizeChanged);
    if (decoder.messageName() == Messages::MediaPlayerPrivateRemote::RenderingModeChanged::name())
        return IPC::handleMessage<Messages::MediaPlayerPrivateRemote::RenderingModeChanged>(decoder, this, &MediaPlayerPrivateRemote::renderingModeChanged);
    if (decoder.messageName() == Messages::MediaPlayerPrivateRemote::CurrentTimeChanged::name())
        return IPC::handleMessage<Messages::MediaPlayerPrivateRemote::CurrentTimeChanged>(decoder, this, &MediaPlayerPrivateRemote::currentTimeChanged);
    if (decoder.messageName() == Messages::MediaPlayerPrivateRemote::AddRemoteAudioTrack::name())
        return IPC::handleMessage<Messages::MediaPlayerPrivateRemote::AddRemoteAudioTrack>(decoder, this, &MediaPlayerPrivateRemote::addRemoteAudioTrack);
    if (decoder.messageName() == Messages::MediaPlayerPrivateRemote::RemoveRemoteAudioTrack::name())
        return IPC::handleMessage<Messages::MediaPlayerPrivateRemote::RemoveRemoteAudioTrack>(decoder, this, &MediaPlayerPrivateRemote::removeRemoteAudioTrack);
    if (decoder.messageName() == Messages::MediaPlayerPrivateRemote::RemoteAudioTrackConfigurationChanged::name())
        return IPC::handleMessage<Messages::MediaPlayerPrivateRemote::RemoteAudioTrackConfigurationChanged>(decoder, this, &MediaPlayerPrivateRemote::remoteAudioTrackConfigurationChanged);
    if (decoder.messageName() == Messages::MediaPlayerPrivateRemote::AddRemoteVideoTrack::name())
        return IPC::handleMessage<Messages::MediaPlayerPrivateRemote::AddRemoteVideoTrack>(decoder, this, &MediaPlayerPrivateRemote::addRemoteVideoTrack);
    if (decoder.messageName() == Messages::MediaPlayerPrivateRemote::RemoveRemoteVideoTrack::name())
        return IPC::handleMessage<Messages::MediaPlayerPrivateRemote::RemoveRemoteVideoTrack>(decoder, this, &MediaPlayerPrivateRemote::removeRemoteVideoTrack);
    if (decoder.messageName() == Messages::MediaPlayerPrivateRemote::RemoteVideoTrackConfigurationChanged::name())
        return IPC::handleMessage<Messages::MediaPlayerPrivateRemote::RemoteVideoTrackConfigurationChanged>(decoder, this, &MediaPlayerPrivateRemote::remoteVideoTrackConfigurationChanged);
    if (decoder.messageName() == Messages::MediaPlayerPrivateRemote::AddRemoteTextTrack::name())
        return IPC::handleMessage<Messages::MediaPlayerPrivateRemote::AddRemoteTextTrack>(decoder, this, &MediaPlayerPrivateRemote::addRemoteTextTrack);
    if (decoder.messageName() == Messages::MediaPlayerPrivateRemote::RemoveRemoteTextTrack::name())
        return IPC::handleMessage<Messages::MediaPlayerPrivateRemote::RemoveRemoteTextTrack>(decoder, this, &MediaPlayerPrivateRemote::removeRemoteTextTrack);
    if (decoder.messageName() == Messages::MediaPlayerPrivateRemote::RemoteTextTrackConfigurationChanged::name())
        return IPC::handleMessage<Messages::MediaPlayerPrivateRemote::RemoteTextTrackConfigurationChanged>(decoder, this, &MediaPlayerPrivateRemote::remoteTextTrackConfigurationChanged);
    if (decoder.messageName() == Messages::MediaPlayerPrivateRemote::ParseWebVTTFileHeader::name())
        return IPC::handleMessage<Messages::MediaPlayerPrivateRemote::ParseWebVTTFileHeader>(decoder, this, &MediaPlayerPrivateRemote::parseWebVTTFileHeader);
    if (decoder.messageName() == Messages::MediaPlayerPrivateRemote::ParseWebVTTCueData::name())
        return IPC::handleMessage<Messages::MediaPlayerPrivateRemote::ParseWebVTTCueData>(decoder, this, &MediaPlayerPrivateRemote::parseWebVTTCueData);
    if (decoder.messageName() == Messages::MediaPlayerPrivateRemote::ParseWebVTTCueDataStruct::name())
        return IPC::handleMessage<Messages::MediaPlayerPrivateRemote::ParseWebVTTCueDataStruct>(decoder, this, &MediaPlayerPrivateRemote::parseWebVTTCueDataStruct);
    if (decoder.messageName() == Messages::MediaPlayerPrivateRemote::AddDataCue::name())
        return IPC::handleMessage<Messages::MediaPlayerPrivateRemote::AddDataCue>(decoder, this, &MediaPlayerPrivateRemote::addDataCue);
#if ENABLE(DATACUE_VALUE)
    if (decoder.messageName() == Messages::MediaPlayerPrivateRemote::AddDataCueWithType::name())
        return IPC::handleMessage<Messages::MediaPlayerPrivateRemote::AddDataCueWithType>(decoder, this, &MediaPlayerPrivateRemote::addDataCueWithType);
#endif
#if ENABLE(DATACUE_VALUE)
    if (decoder.messageName() == Messages::MediaPlayerPrivateRemote::UpdateDataCue::name())
        return IPC::handleMessage<Messages::MediaPlayerPrivateRemote::UpdateDataCue>(decoder, this, &MediaPlayerPrivateRemote::updateDataCue);
#endif
#if ENABLE(DATACUE_VALUE)
    if (decoder.messageName() == Messages::MediaPlayerPrivateRemote::RemoveDataCue::name())
        return IPC::handleMessage<Messages::MediaPlayerPrivateRemote::RemoveDataCue>(decoder, this, &MediaPlayerPrivateRemote::removeDataCue);
#endif
    if (decoder.messageName() == Messages::MediaPlayerPrivateRemote::AddGenericCue::name())
        return IPC::handleMessage<Messages::MediaPlayerPrivateRemote::AddGenericCue>(decoder, this, &MediaPlayerPrivateRemote::addGenericCue);
    if (decoder.messageName() == Messages::MediaPlayerPrivateRemote::UpdateGenericCue::name())
        return IPC::handleMessage<Messages::MediaPlayerPrivateRemote::UpdateGenericCue>(decoder, this, &MediaPlayerPrivateRemote::updateGenericCue);
    if (decoder.messageName() == Messages::MediaPlayerPrivateRemote::RemoveGenericCue::name())
        return IPC::handleMessage<Messages::MediaPlayerPrivateRemote::RemoveGenericCue>(decoder, this, &MediaPlayerPrivateRemote::removeGenericCue);
    if (decoder.messageName() == Messages::MediaPlayerPrivateRemote::RequestResource::name())
        return IPC::handleMessageAsync<Messages::MediaPlayerPrivateRemote::RequestResource>(connection, decoder, this, &MediaPlayerPrivateRemote::requestResource);
    if (decoder.messageName() == Messages::MediaPlayerPrivateRemote::RemoveResource::name())
        return IPC::handleMessage<Messages::MediaPlayerPrivateRemote::RemoveResource>(decoder, this, &MediaPlayerPrivateRemote::removeResource);
    if (decoder.messageName() == Messages::MediaPlayerPrivateRemote::SendH2Ping::name())
        return IPC::handleMessageAsync<Messages::MediaPlayerPrivateRemote::SendH2Ping>(connection, decoder, this, &MediaPlayerPrivateRemote::sendH2Ping);
    if (decoder.messageName() == Messages::MediaPlayerPrivateRemote::ResourceNotSupported::name())
        return IPC::handleMessage<Messages::MediaPlayerPrivateRemote::ResourceNotSupported>(decoder, this, &MediaPlayerPrivateRemote::resourceNotSupported);
    if (decoder.messageName() == Messages::MediaPlayerPrivateRemote::ActiveSourceBuffersChanged::name())
        return IPC::handleMessage<Messages::MediaPlayerPrivateRemote::ActiveSourceBuffersChanged>(decoder, this, &MediaPlayerPrivateRemote::activeSourceBuffersChanged);
#if ENABLE(ENCRYPTED_MEDIA)
    if (decoder.messageName() == Messages::MediaPlayerPrivateRemote::WaitingForKeyChanged::name())
        return IPC::handleMessage<Messages::MediaPlayerPrivateRemote::WaitingForKeyChanged>(decoder, this, &MediaPlayerPrivateRemote::waitingForKeyChanged);
#endif
#if ENABLE(ENCRYPTED_MEDIA)
    if (decoder.messageName() == Messages::MediaPlayerPrivateRemote::InitializationDataEncountered::name())
        return IPC::handleMessage<Messages::MediaPlayerPrivateRemote::InitializationDataEncountered>(decoder, this, &MediaPlayerPrivateRemote::initializationDataEncountered);
#endif
#if ENABLE(LEGACY_ENCRYPTED_MEDIA)
    if (decoder.messageName() == Messages::MediaPlayerPrivateRemote::MediaPlayerKeyNeeded::name())
        return IPC::handleMessage<Messages::MediaPlayerPrivateRemote::MediaPlayerKeyNeeded>(decoder, this, &MediaPlayerPrivateRemote::mediaPlayerKeyNeeded);
#endif
#if ENABLE(WIRELESS_PLAYBACK_TARGET)
    if (decoder.messageName() == Messages::MediaPlayerPrivateRemote::CurrentPlaybackTargetIsWirelessChanged::name())
        return IPC::handleMessage<Messages::MediaPlayerPrivateRemote::CurrentPlaybackTargetIsWirelessChanged>(decoder, this, &MediaPlayerPrivateRemote::currentPlaybackTargetIsWirelessChanged);
#endif
#if PLATFORM(IOS_FAMILY)
    if (decoder.messageName() == Messages::MediaPlayerPrivateRemote::GetRawCookies::name())
        return IPC::handleMessageAsync<Messages::MediaPlayerPrivateRemote::GetRawCookies>(connection, decoder, this, &MediaPlayerPrivateRemote::getRawCookies);
#endif
    UNUSED_PARAM(connection);
    UNUSED_PARAM(decoder);
#if ENABLE(IPC_TESTING_API)
    if (connection.ignoreInvalidMessageForTesting())
        return;
#endif // ENABLE(IPC_TESTING_API)
    ASSERT_NOT_REACHED_WITH_MESSAGE("Unhandled message %s to %" PRIu64, description(decoder.messageName()), decoder.destinationID());
}

} // namespace WebKit

#endif // ENABLE(GPU_PROCESS)
