/*
 * Copyright (C) 2021 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "config.h"
#if ENABLE(NETSCAPE_PLUGIN_API)
#include "PluginProxy.h"

#include "ArgumentCoders.h"
#include "Decoder.h"
#include "HandleMessage.h"
#include "NPVariantData.h"
#include "PluginProxyMessages.h"
#include "WebCoreArgumentCoders.h"
#include <WebCore/HTTPHeaderMap.h>
#include <WebCore/IntRect.h>
#include <WebCore/ProtectionSpace.h>
#include <wtf/Vector.h>
#include <wtf/text/WTFString.h>

namespace Messages {

namespace PluginProxy {

void ProxiesForURL::send(UniqueRef<IPC::Encoder>&& encoder, IPC::Connection& connection, const String& proxiesString)
{
    encoder.get() << proxiesString;
    connection.sendSyncReply(WTFMove(encoder));
}

void CookiesForURL::send(UniqueRef<IPC::Encoder>&& encoder, IPC::Connection& connection, const String& cookieString)
{
    encoder.get() << cookieString;
    connection.sendSyncReply(WTFMove(encoder));
}

void GetAuthenticationInfo::send(UniqueRef<IPC::Encoder>&& encoder, IPC::Connection& connection, bool returnValue, const String& username, const String& password)
{
    encoder.get() << returnValue;
    encoder.get() << username;
    encoder.get() << password;
    connection.sendSyncReply(WTFMove(encoder));
}

void GetPluginElementNPObject::send(UniqueRef<IPC::Encoder>&& encoder, IPC::Connection& connection, uint64_t pluginElementNPObjectID)
{
    encoder.get() << pluginElementNPObjectID;
    connection.sendSyncReply(WTFMove(encoder));
}

void Evaluate::send(UniqueRef<IPC::Encoder>&& encoder, IPC::Connection& connection, bool returnValue, const WebKit::NPVariantData& resultData)
{
    encoder.get() << returnValue;
    encoder.get() << resultData;
    connection.sendSyncReply(WTFMove(encoder));
}

#if PLATFORM(X11)

void CreatePluginContainer::send(UniqueRef<IPC::Encoder>&& encoder, IPC::Connection& connection, uint64_t windowID)
{
    encoder.get() << windowID;
    connection.sendSyncReply(WTFMove(encoder));
}

#endif

void DidCreatePlugin::send(UniqueRef<IPC::Encoder>&& encoder, IPC::Connection& connection)
{
    connection.sendSyncReply(WTFMove(encoder));
}

void DidFailToCreatePlugin::send(UniqueRef<IPC::Encoder>&& encoder, IPC::Connection& connection)
{
    connection.sendSyncReply(WTFMove(encoder));
}

} // namespace PluginProxy

} // namespace Messages

namespace WebKit {

void PluginProxy::didReceivePluginProxyMessage(IPC::Connection& connection, IPC::Decoder& decoder)
{
    auto protectedThis = makeRef(*this);
    if (decoder.messageName() == Messages::PluginProxy::LoadURL::name())
        return IPC::handleMessage<Messages::PluginProxy::LoadURL>(decoder, this, &PluginProxy::loadURL);
    if (decoder.messageName() == Messages::PluginProxy::Update::name())
        return IPC::handleMessage<Messages::PluginProxy::Update>(decoder, this, &PluginProxy::update);
    if (decoder.messageName() == Messages::PluginProxy::SetCookiesForURL::name())
        return IPC::handleMessage<Messages::PluginProxy::SetCookiesForURL>(decoder, this, &PluginProxy::setCookiesForURL);
    if (decoder.messageName() == Messages::PluginProxy::CancelStreamLoad::name())
        return IPC::handleMessage<Messages::PluginProxy::CancelStreamLoad>(decoder, this, &PluginProxy::cancelStreamLoad);
    if (decoder.messageName() == Messages::PluginProxy::ContinueStreamLoad::name())
        return IPC::handleMessage<Messages::PluginProxy::ContinueStreamLoad>(decoder, this, &PluginProxy::continueStreamLoad);
    if (decoder.messageName() == Messages::PluginProxy::CancelManualStreamLoad::name())
        return IPC::handleMessage<Messages::PluginProxy::CancelManualStreamLoad>(decoder, this, &PluginProxy::cancelManualStreamLoad);
    if (decoder.messageName() == Messages::PluginProxy::SetStatusbarText::name())
        return IPC::handleMessage<Messages::PluginProxy::SetStatusbarText>(decoder, this, &PluginProxy::setStatusbarText);
#if PLATFORM(COCOA)
    if (decoder.messageName() == Messages::PluginProxy::PluginFocusOrWindowFocusChanged::name())
        return IPC::handleMessage<Messages::PluginProxy::PluginFocusOrWindowFocusChanged>(decoder, this, &PluginProxy::pluginFocusOrWindowFocusChanged);
#endif
#if PLATFORM(COCOA)
    if (decoder.messageName() == Messages::PluginProxy::SetComplexTextInputState::name())
        return IPC::handleMessage<Messages::PluginProxy::SetComplexTextInputState>(decoder, this, &PluginProxy::setComplexTextInputState);
#endif
#if PLATFORM(COCOA)
    if (decoder.messageName() == Messages::PluginProxy::SetLayerHostingContextID::name())
        return IPC::handleMessage<Messages::PluginProxy::SetLayerHostingContextID>(decoder, this, &PluginProxy::setLayerHostingContextID);
#endif
#if PLATFORM(X11)
    if (decoder.messageName() == Messages::PluginProxy::WindowedPluginGeometryDidChange::name())
        return IPC::handleMessage<Messages::PluginProxy::WindowedPluginGeometryDidChange>(decoder, this, &PluginProxy::windowedPluginGeometryDidChange);
#endif
#if PLATFORM(X11)
    if (decoder.messageName() == Messages::PluginProxy::WindowedPluginVisibilityDidChange::name())
        return IPC::handleMessage<Messages::PluginProxy::WindowedPluginVisibilityDidChange>(decoder, this, &PluginProxy::windowedPluginVisibilityDidChange);
#endif
    if (decoder.messageName() == Messages::PluginProxy::SetPluginIsPlayingAudio::name())
        return IPC::handleMessage<Messages::PluginProxy::SetPluginIsPlayingAudio>(decoder, this, &PluginProxy::setPluginIsPlayingAudio);
    UNUSED_PARAM(connection);
    UNUSED_PARAM(decoder);
#if ENABLE(IPC_TESTING_API)
    if (connection.ignoreInvalidMessageForTesting())
        return;
#endif // ENABLE(IPC_TESTING_API)
    ASSERT_NOT_REACHED_WITH_MESSAGE("Unhandled message %s to %" PRIu64, description(decoder.messageName()), decoder.destinationID());
}

bool PluginProxy::didReceiveSyncPluginProxyMessage(IPC::Connection& connection, IPC::Decoder& decoder, UniqueRef<IPC::Encoder>& replyEncoder)
{
    auto protectedThis = makeRef(*this);
    if (decoder.messageName() == Messages::PluginProxy::ProxiesForURL::name())
        return IPC::handleMessageSynchronous<Messages::PluginProxy::ProxiesForURL>(connection, decoder, replyEncoder, this, &PluginProxy::proxiesForURL);
    if (decoder.messageName() == Messages::PluginProxy::CookiesForURL::name())
        return IPC::handleMessageSynchronous<Messages::PluginProxy::CookiesForURL>(connection, decoder, replyEncoder, this, &PluginProxy::cookiesForURL);
    if (decoder.messageName() == Messages::PluginProxy::GetAuthenticationInfo::name())
        return IPC::handleMessageSynchronous<Messages::PluginProxy::GetAuthenticationInfo>(connection, decoder, replyEncoder, this, &PluginProxy::getAuthenticationInfo);
    if (decoder.messageName() == Messages::PluginProxy::GetPluginElementNPObject::name())
        return IPC::handleMessageSynchronous<Messages::PluginProxy::GetPluginElementNPObject>(connection, decoder, replyEncoder, this, &PluginProxy::getPluginElementNPObject);
    if (decoder.messageName() == Messages::PluginProxy::Evaluate::name())
        return IPC::handleMessageSynchronous<Messages::PluginProxy::Evaluate>(connection, decoder, replyEncoder, this, &PluginProxy::evaluate);
#if PLATFORM(X11)
    if (decoder.messageName() == Messages::PluginProxy::CreatePluginContainer::name())
        return IPC::handleMessageSynchronous<Messages::PluginProxy::CreatePluginContainer>(connection, decoder, replyEncoder, this, &PluginProxy::createPluginContainer);
#endif
    if (decoder.messageName() == Messages::PluginProxy::DidCreatePlugin::name())
        return IPC::handleMessageSynchronous<Messages::PluginProxy::DidCreatePlugin>(connection, decoder, replyEncoder, this, &PluginProxy::didCreatePlugin);
    if (decoder.messageName() == Messages::PluginProxy::DidFailToCreatePlugin::name())
        return IPC::handleMessageSynchronous<Messages::PluginProxy::DidFailToCreatePlugin>(connection, decoder, replyEncoder, this, &PluginProxy::didFailToCreatePlugin);
    UNUSED_PARAM(connection);
    UNUSED_PARAM(decoder);
    UNUSED_PARAM(replyEncoder);
#if ENABLE(IPC_TESTING_API)
    if (connection.ignoreInvalidMessageForTesting())
        return false;
#endif // ENABLE(IPC_TESTING_API)
    ASSERT_NOT_REACHED_WITH_MESSAGE("Unhandled synchronous message %s to %" PRIu64, description(decoder.messageName()), decoder.destinationID());
    return false;
}

} // namespace WebKit

#endif // ENABLE(NETSCAPE_PLUGIN_API)
