/*
 * Copyright (C) 2021 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "config.h"
#if PLATFORM(COCOA) && ENABLE(GPU_PROCESS) && ENABLE(MEDIA_STREAM) && HAVE(AVASSETWRITERDELEGATE)
#include "RemoteMediaRecorder.h"

#include "DataReference.h"
#include "Decoder.h"
#include "HandleMessage.h"
#include "RemoteMediaRecorderMessages.h"
#include "SharedMemory.h"
#include "WebCoreArgumentCoders.h"
#include <WebCore/CAAudioStreamDescription.h>
#include <WebCore/RemoteVideoSample.h>
#include <wtf/MediaTime.h>

namespace Messages {

namespace RemoteMediaRecorder {

void FetchData::callReply(IPC::Decoder& decoder, CompletionHandler<void(IPC::DataReference&&, double&&)>&& completionHandler)
{
    std::optional<IPC::DataReference> buffer;
    decoder >> buffer;
    if (!buffer) {
        ASSERT_NOT_REACHED();
        cancelReply(WTFMove(completionHandler));
        return;
    }
    std::optional<double> timeCode;
    decoder >> timeCode;
    if (!timeCode) {
        ASSERT_NOT_REACHED();
        cancelReply(WTFMove(completionHandler));
        return;
    }
    completionHandler(WTFMove(*buffer), WTFMove(*timeCode));
}

void FetchData::cancelReply(CompletionHandler<void(IPC::DataReference&&, double&&)>&& completionHandler)
{
    completionHandler(IPC::AsyncReplyError<IPC::DataReference>::create(), IPC::AsyncReplyError<double>::create());
}

void FetchData::send(UniqueRef<IPC::Encoder>&& encoder, IPC::Connection& connection, const IPC::DataReference& buffer, double timeCode)
{
    encoder.get() << buffer;
    encoder.get() << timeCode;
    connection.sendSyncReply(WTFMove(encoder));
}

void StopRecording::callReply(IPC::Decoder& decoder, CompletionHandler<void()>&& completionHandler)
{
    completionHandler();
}

void StopRecording::cancelReply(CompletionHandler<void()>&& completionHandler)
{
    completionHandler();
}

void StopRecording::send(UniqueRef<IPC::Encoder>&& encoder, IPC::Connection& connection)
{
    connection.sendSyncReply(WTFMove(encoder));
}

void Pause::callReply(IPC::Decoder& decoder, CompletionHandler<void()>&& completionHandler)
{
    completionHandler();
}

void Pause::cancelReply(CompletionHandler<void()>&& completionHandler)
{
    completionHandler();
}

void Pause::send(UniqueRef<IPC::Encoder>&& encoder, IPC::Connection& connection)
{
    connection.sendSyncReply(WTFMove(encoder));
}

void Resume::callReply(IPC::Decoder& decoder, CompletionHandler<void()>&& completionHandler)
{
    completionHandler();
}

void Resume::cancelReply(CompletionHandler<void()>&& completionHandler)
{
    completionHandler();
}

void Resume::send(UniqueRef<IPC::Encoder>&& encoder, IPC::Connection& connection)
{
    connection.sendSyncReply(WTFMove(encoder));
}

} // namespace RemoteMediaRecorder

} // namespace Messages

namespace WebKit {

void RemoteMediaRecorder::didReceiveMessage(IPC::Connection& connection, IPC::Decoder& decoder)
{
    if (decoder.messageName() == Messages::RemoteMediaRecorder::AudioSamplesStorageChanged::name())
        return IPC::handleMessage<Messages::RemoteMediaRecorder::AudioSamplesStorageChanged>(decoder, this, &RemoteMediaRecorder::audioSamplesStorageChanged);
    if (decoder.messageName() == Messages::RemoteMediaRecorder::AudioSamplesAvailable::name())
        return IPC::handleMessage<Messages::RemoteMediaRecorder::AudioSamplesAvailable>(decoder, this, &RemoteMediaRecorder::audioSamplesAvailable);
    if (decoder.messageName() == Messages::RemoteMediaRecorder::VideoSampleAvailable::name())
        return IPC::handleMessage<Messages::RemoteMediaRecorder::VideoSampleAvailable>(decoder, this, &RemoteMediaRecorder::videoSampleAvailable);
    if (decoder.messageName() == Messages::RemoteMediaRecorder::FetchData::name())
        return IPC::handleMessageAsync<Messages::RemoteMediaRecorder::FetchData>(connection, decoder, this, &RemoteMediaRecorder::fetchData);
    if (decoder.messageName() == Messages::RemoteMediaRecorder::StopRecording::name())
        return IPC::handleMessageAsync<Messages::RemoteMediaRecorder::StopRecording>(connection, decoder, this, &RemoteMediaRecorder::stopRecording);
    if (decoder.messageName() == Messages::RemoteMediaRecorder::Pause::name())
        return IPC::handleMessageAsync<Messages::RemoteMediaRecorder::Pause>(connection, decoder, this, &RemoteMediaRecorder::pause);
    if (decoder.messageName() == Messages::RemoteMediaRecorder::Resume::name())
        return IPC::handleMessageAsync<Messages::RemoteMediaRecorder::Resume>(connection, decoder, this, &RemoteMediaRecorder::resume);
    UNUSED_PARAM(connection);
    UNUSED_PARAM(decoder);
#if ENABLE(IPC_TESTING_API)
    if (connection.ignoreInvalidMessageForTesting())
        return;
#endif // ENABLE(IPC_TESTING_API)
    ASSERT_NOT_REACHED_WITH_MESSAGE("Unhandled message %s to %" PRIu64, description(decoder.messageName()), decoder.destinationID());
}

} // namespace WebKit

#endif // PLATFORM(COCOA) && ENABLE(GPU_PROCESS) && ENABLE(MEDIA_STREAM) && HAVE(AVASSETWRITERDELEGATE)
