/*
 * Copyright (C) 2021 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "config.h"
#if ENABLE(GPU_PROCESS)
#include "RemoteRenderingBackend.h"

#include "ArgumentCoders.h"
#include "Decoder.h"
#include "FontReference.h"
#include "GPUProcessWakeupMessageArguments.h"
#include "HandleMessage.h"
#include "IPCSemaphore.h"
#include "RemoteRenderingBackendMessages.h"
#include "RenderingUpdateID.h"
#include "ShareableBitmap.h"
#include "SharedMemory.h"
#include "WebCoreArgumentCoders.h"
#include <WebCore/ColorSpace.h>
#include <WebCore/DisplayList.h>
#include <WebCore/FloatSize.h>
#include <WebCore/ImageBufferBackend.h>
#include <WebCore/RenderingMode.h>
#include <WebCore/RenderingResourceIdentifier.h>
#include <optional>
#include <wtf/Vector.h>
#include <wtf/text/WTFString.h>

namespace Messages {

namespace RemoteRenderingBackend {

void UpdateSharedMemoryForGetPixelBuffer::send(UniqueRef<IPC::Encoder>&& encoder, IPC::Connection& connection, const WebKit::SharedMemory::IPCHandle& handle)
{
    encoder.get() << handle;
    connection.sendSyncReply(WTFMove(encoder));
}

void SemaphoreForGetPixelBuffer::send(UniqueRef<IPC::Encoder>&& encoder, IPC::Connection& connection, const IPC::Semaphore& semaphore)
{
    encoder.get() << semaphore;
    connection.sendSyncReply(WTFMove(encoder));
}

void UpdateSharedMemoryAndSemaphoreForGetPixelBuffer::send(UniqueRef<IPC::Encoder>&& encoder, IPC::Connection& connection, const WebKit::SharedMemory::IPCHandle& handle, const IPC::Semaphore& semaphore)
{
    encoder.get() << handle;
    encoder.get() << semaphore;
    connection.sendSyncReply(WTFMove(encoder));
}

void GetDataURLForImageBuffer::send(UniqueRef<IPC::Encoder>&& encoder, IPC::Connection& connection, const String& urlString)
{
    encoder.get() << urlString;
    connection.sendSyncReply(WTFMove(encoder));
}

void GetDataForImageBuffer::send(UniqueRef<IPC::Encoder>&& encoder, IPC::Connection& connection, const Vector<uint8_t>& data)
{
    encoder.get() << data;
    connection.sendSyncReply(WTFMove(encoder));
}

void GetShareableBitmapForImageBuffer::send(UniqueRef<IPC::Encoder>&& encoder, IPC::Connection& connection, const WebKit::ShareableBitmap::Handle& handle)
{
    encoder.get() << handle;
    connection.sendSyncReply(WTFMove(encoder));
}

} // namespace RemoteRenderingBackend

} // namespace Messages

namespace WebKit {

void RemoteRenderingBackend::didReceiveMessage(IPC::Connection& connection, IPC::Decoder& decoder)
{
    auto protectedThis = makeRef(*this);
    if (decoder.messageName() == Messages::RemoteRenderingBackend::CreateImageBuffer::name())
        return IPC::handleMessage<Messages::RemoteRenderingBackend::CreateImageBuffer>(decoder, this, &RemoteRenderingBackend::createImageBuffer);
    if (decoder.messageName() == Messages::RemoteRenderingBackend::WakeUpAndApplyDisplayList::name())
        return IPC::handleMessage<Messages::RemoteRenderingBackend::WakeUpAndApplyDisplayList>(decoder, this, &RemoteRenderingBackend::wakeUpAndApplyDisplayList);
    if (decoder.messageName() == Messages::RemoteRenderingBackend::DestroyGetPixelBufferSharedMemory::name())
        return IPC::handleMessage<Messages::RemoteRenderingBackend::DestroyGetPixelBufferSharedMemory>(decoder, this, &RemoteRenderingBackend::destroyGetPixelBufferSharedMemory);
    if (decoder.messageName() == Messages::RemoteRenderingBackend::CacheNativeImage::name())
        return IPC::handleMessage<Messages::RemoteRenderingBackend::CacheNativeImage>(decoder, this, &RemoteRenderingBackend::cacheNativeImage);
    if (decoder.messageName() == Messages::RemoteRenderingBackend::CacheFont::name())
        return IPC::handleMessage<Messages::RemoteRenderingBackend::CacheFont>(decoder, this, &RemoteRenderingBackend::cacheFont);
    if (decoder.messageName() == Messages::RemoteRenderingBackend::DeleteAllFonts::name())
        return IPC::handleMessage<Messages::RemoteRenderingBackend::DeleteAllFonts>(decoder, this, &RemoteRenderingBackend::deleteAllFonts);
    if (decoder.messageName() == Messages::RemoteRenderingBackend::DidCreateSharedDisplayListHandle::name())
        return IPC::handleMessage<Messages::RemoteRenderingBackend::DidCreateSharedDisplayListHandle>(decoder, this, &RemoteRenderingBackend::didCreateSharedDisplayListHandle);
    if (decoder.messageName() == Messages::RemoteRenderingBackend::ReleaseRemoteResource::name())
        return IPC::handleMessage<Messages::RemoteRenderingBackend::ReleaseRemoteResource>(decoder, this, &RemoteRenderingBackend::releaseRemoteResource);
    if (decoder.messageName() == Messages::RemoteRenderingBackend::FinalizeRenderingUpdate::name())
        return IPC::handleMessage<Messages::RemoteRenderingBackend::FinalizeRenderingUpdate>(decoder, this, &RemoteRenderingBackend::finalizeRenderingUpdate);
    UNUSED_PARAM(connection);
    UNUSED_PARAM(decoder);
#if ENABLE(IPC_TESTING_API)
    if (connection.ignoreInvalidMessageForTesting())
        return;
#endif // ENABLE(IPC_TESTING_API)
    ASSERT_NOT_REACHED_WITH_MESSAGE("Unhandled message %s to %" PRIu64, description(decoder.messageName()), decoder.destinationID());
}

bool RemoteRenderingBackend::didReceiveSyncMessage(IPC::Connection& connection, IPC::Decoder& decoder, UniqueRef<IPC::Encoder>& replyEncoder)
{
    auto protectedThis = makeRef(*this);
    if (decoder.messageName() == Messages::RemoteRenderingBackend::UpdateSharedMemoryForGetPixelBuffer::name())
        return IPC::handleMessageSynchronous<Messages::RemoteRenderingBackend::UpdateSharedMemoryForGetPixelBuffer>(connection, decoder, replyEncoder, this, &RemoteRenderingBackend::updateSharedMemoryForGetPixelBuffer);
    if (decoder.messageName() == Messages::RemoteRenderingBackend::SemaphoreForGetPixelBuffer::name())
        return IPC::handleMessageSynchronous<Messages::RemoteRenderingBackend::SemaphoreForGetPixelBuffer>(connection, decoder, replyEncoder, this, &RemoteRenderingBackend::semaphoreForGetPixelBuffer);
    if (decoder.messageName() == Messages::RemoteRenderingBackend::UpdateSharedMemoryAndSemaphoreForGetPixelBuffer::name())
        return IPC::handleMessageSynchronous<Messages::RemoteRenderingBackend::UpdateSharedMemoryAndSemaphoreForGetPixelBuffer>(connection, decoder, replyEncoder, this, &RemoteRenderingBackend::updateSharedMemoryAndSemaphoreForGetPixelBuffer);
    if (decoder.messageName() == Messages::RemoteRenderingBackend::GetDataURLForImageBuffer::name())
        return IPC::handleMessageSynchronous<Messages::RemoteRenderingBackend::GetDataURLForImageBuffer>(connection, decoder, replyEncoder, this, &RemoteRenderingBackend::getDataURLForImageBuffer);
    if (decoder.messageName() == Messages::RemoteRenderingBackend::GetDataForImageBuffer::name())
        return IPC::handleMessageSynchronous<Messages::RemoteRenderingBackend::GetDataForImageBuffer>(connection, decoder, replyEncoder, this, &RemoteRenderingBackend::getDataForImageBuffer);
    if (decoder.messageName() == Messages::RemoteRenderingBackend::GetShareableBitmapForImageBuffer::name())
        return IPC::handleMessageSynchronous<Messages::RemoteRenderingBackend::GetShareableBitmapForImageBuffer>(connection, decoder, replyEncoder, this, &RemoteRenderingBackend::getShareableBitmapForImageBuffer);
    UNUSED_PARAM(connection);
    UNUSED_PARAM(decoder);
    UNUSED_PARAM(replyEncoder);
#if ENABLE(IPC_TESTING_API)
    if (connection.ignoreInvalidMessageForTesting())
        return false;
#endif // ENABLE(IPC_TESTING_API)
    ASSERT_NOT_REACHED_WITH_MESSAGE("Unhandled synchronous message %s to %" PRIu64, description(decoder.messageName()), decoder.destinationID());
    return false;
}

} // namespace WebKit

#endif // ENABLE(GPU_PROCESS)
